@@ -74,12 +74,13 @@ _vectorized_sgp4(PyObject *args, elsetrec *raw_satrec_array, int imax)
             for (Py_ssize_t j=0; j < jmax; j++) {
                 double t = (jd[j] - satrec.jdsatepoch) * 1440.0
                          + (fr[j] - satrec.jdsatepochF) * 1440.0;
-                Py_ssize_t k = i * jmax + j;
-                SGP4Funcs::sgp4(satrec, t, r + k*3, v + k*3);
+                Py_ssize_t k1 = i * jmax + j;
+                Py_ssize_t k3 = 3 * k1;
+                SGP4Funcs::sgp4(satrec, t, r + k3, v + k3);
+                e[k1] = (uint8_t) satrec.error;
                 if (satrec.error && satrec.error < 6) {
-                    r[k] = r[k+1] = r[k+2] = v[k] = v[k+1] = v[k+2] = NAN;
+                    r[k3] = r[k3+1] = r[k3+2] = v[k3] = v[k3+1] = v[k3+2] = NAN;
                 }
-                e[k] = (uint8_t) satrec.error;
             }
         }
     }
@@ -119,6 +120,11 @@ Satrec_twoline2rv(PyTypeObject *cls, PyObject *args)
     line1[68] = '\0';
     line2[68] = '\0';
 
+    /* Allocate the new object. */
+    SatrecObject *self = (SatrecObject*) cls->tp_alloc(cls, 0);
+    if (!self)
+        return NULL;
+
     /* Correct for locales that use a comma as the decimal point, since
        users report that the scanf() function on macOS is sensitive to
        locale when parsing floats.  This operation is not thread-safe,
@@ -131,10 +137,18 @@ Satrec_twoline2rv(PyTypeObject *cls, PyObject *args)
     if (switch_locale)
         old_locale = setlocale(LC_NUMERIC, "C");
 
-    SatrecObject *self = (SatrecObject*) cls->tp_alloc(cls, 0);
-    if (!self)
-        return NULL;
+    /* Leading spaces in a catalog number make scanf() in the official
+       code consume the Classification letter as part of the catalog
+       number.  (The first character of the International Designator
+       then gets consumed as the Classification instead.)  But no
+       parsing error is reported, which is bad for users, so let's avoid
+       the situation by adding leading zeros ourselves. */
+    for (int i=2; i<7; i++) {
+        if (line1[i] == ' ') line1[i] = '0';
+        if (line2[i] == ' ') line2[i] = '0';
+    }
 
+    /* Call the official routine. */
     SGP4Funcs::twoline2rv(line1, line2, ' ', ' ', 'i', whichconst,
                           dummy, dummy, dummy, self->satrec);
 
@@ -192,14 +206,27 @@ Satrec_sgp4init(PyObject *self, PyObject *args)
                         nodeo, satrec);
 
     /* Populate date fields that SGP4Funcs::twoline2rv would set. */
+    double whole;
+    double fraction = modf(epoch, &whole);
+    double whole_jd = whole + 2433281.5;
+
+    /* Go out on a limb: if `epoch` has no decimal digits past the 8
+       decimal places stored in a TLE, then assume the user is trying
+       to specify an exact decimal fraction. */
+    double epoch8 = epoch * 1e8;
+    if (round(epoch8) == epoch8) {
+        fraction = round(fraction * 1e8) / 1e8;
+    }
+
+    satrec.jdsatepoch = whole_jd;
+    satrec.jdsatepochF = fraction;
+
     int y, m, d, H, M;
     double S, jan0jd, jan0fr /* always comes out 0.0 */;
-    SGP4Funcs::invjday_SGP4(2433281.5, epoch, y, m, d, H, M, S);
+    SGP4Funcs::invjday_SGP4(2433281.5, whole, y, m, d, H, M, S);
     SGP4Funcs::jday_SGP4(y, 1, 0, 0, 0, 0.0, jan0jd, jan0fr);
-    satrec.epochyr = y % 1000;
-    satrec.epochdays = 2433281.5 - jan0jd + epoch;
-    satrec.jdsatepochF = modf(epoch, &satrec.jdsatepoch);
-    satrec.jdsatepoch += 2433281.5;
+    satrec.epochyr = y % 100;
+    satrec.epochdays = whole_jd - jan0jd + fraction;
 
     /* Return true as sgp4init does, satrec.error contains any error codes */
 
@@ -268,16 +295,16 @@ static PyMemberDef Satrec_members[] = {
 
     {"operationmode", T_CHAR, O(operationmode), READONLY,
      PyDoc_STR("Operation mode: 'a' legacy AFSPC, or 'i' improved.")},
-    {"jdsatepoch", T_DOUBLE, O(jdsatepoch), READONLY,
+    {"jdsatepoch", T_DOUBLE, O(jdsatepoch), 0,
      PyDoc_STR("Julian date of epoch, day number (see jdsatepochF).")},
-    {"jdsatepochF", T_DOUBLE, O(jdsatepochF), READONLY,
+    {"jdsatepochF", T_DOUBLE, O(jdsatepochF), 0,
      PyDoc_STR("Julian date of epoch, fraction of day (see jdsatepoch).")},
     {"classification", T_CHAR, O(classification), 0,
      "Usually U=Unclassified, C=Classified, or S=Secret."},
     /* intldesg: inline character array; see Satrec_getset. */
-    {"epochyr", T_INT, O(epochyr), READONLY,
+    {"epochyr", T_INT, O(epochyr), 0,
      PyDoc_STR("Year of this element set's epoch (see epochdays). Not set by sgp4init().")},
-    {"epochdays", T_DOUBLE, O(epochdays), READONLY,
+    {"epochdays", T_DOUBLE, O(epochdays), 0,
      PyDoc_STR("Day of the year of this element set's epoch (see epochyr). Not set by sgp4init().")},
     {"ndot", T_DOUBLE, O(ndot), READONLY,
      PyDoc_STR("Ballistic Coefficient in revs/day.")},
@@ -314,7 +341,7 @@ static PyMemberDef Satrec_members[] = {
 
     {"method", T_CHAR, O(method), READONLY,
      PyDoc_STR("Method, either 'n' near earth or 'd' deep space.")},
-    {"error", T_INT, O(method), READONLY,
+    {"error", T_INT, O(error), READONLY,
      PyDoc_STR("Error code (1-6) documented in sgp4()")},
     {"a", T_DOUBLE, O(a), READONLY,
      PyDoc_STR("semi-major axis")},
